package passbaseplugin.pettersonapps.com.passbase_flutter

import android.app.Activity
import android.content.Context
import android.view.View
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.Registrar
import com.passbase.passbase_sdk.PassbaseSDK
import com.passbase.passbase_sdk.PassbaseButton
import com.passbase.passbase_sdk.PassbaseSDKListener
import io.flutter.plugin.common.StandardMessageCodec
import io.flutter.plugin.platform.PlatformView
import io.flutter.plugin.platform.PlatformViewFactory

class PassbaseFlutterPlugin(private val passbase: PassbaseSDK, private val channel: MethodChannel) : MethodCallHandler {

    companion object {
        @JvmStatic
        fun registerWith(registrar: Registrar) {
            val channel = MethodChannel(registrar.messenger(), "passbase_flutter")
            val passbase = PassbaseSDK(registrar.activity())
            channel.setMethodCallHandler(PassbaseFlutterPlugin(passbase, channel))
            PassbaseButtonPlugin.registerWith(registrar, passbase, channel)
        }
    }

    override fun onMethodCall(call: MethodCall, result: Result) {
        when (call.method) {
            "initialize" -> {
                PassbaseSDK.source = 3
                passbase.initialize(call.arguments as String)

                result.success(null)
            }
            "startVerification" -> {
                passbase.callback(object : PassbaseSDKListener {
                    override fun onStart() {
                        channel.invokeMethod("verification_start", "Verification started")
                    }

                    override fun onFinish(identityAccessKey: String) {
                        channel.invokeMethod("verification_complete", identityAccessKey)
                    }

                    override fun onError(errorCode: String) {
                        channel.invokeMethod("verification_error", errorCode)
                    }

                    override fun onSubmitted(identityAccessKey: String) {
                        channel.invokeMethod("verification_submitted", identityAccessKey)
                    }
                })

                passbase.startVerification()
            }
            "setPrefillUserEmail" -> {
                passbase.prefillUserEmail = call.arguments as String?
            }
            "setMetaData" -> {
                passbase.metaData = call.arguments as String?
            }
            "setPrefillCountry" -> {
                passbase.prefillCountry = call.arguments as String?
            }
            else -> {
                result.notImplemented()
            }
        }
    }
}

object PassbaseButtonPlugin {
    fun registerWith(registrar: Registrar, passbase: PassbaseSDK, channel: MethodChannel) {
        val viewFactory = PassbaseButtonFactory(registrar.activity(), passbase, channel)
        registrar.platformViewRegistry()
                .registerViewFactory("PassbaseButton", viewFactory)
    }
}

class PassbaseButtonFactory(private val activity: Activity, private val passbase: PassbaseSDK, private val channel: MethodChannel) :
        PlatformViewFactory(StandardMessageCodec.INSTANCE) {

    override fun create(context: Context,
                        id: Int,
                        args: Any?): PlatformView {
        return PassbaseButtonContainer(activity as Context,
                passbase,
                channel)
    }
}

class PassbaseButtonContainer internal constructor(context: Context, private val passbase: PassbaseSDK, private val channel: MethodChannel) : PlatformView {
    private val passbaseButton: PassbaseButton = PassbaseButton(context = context)

    init {
        passbaseButton.setOnClickListener {
            passbase.callback(object : PassbaseSDKListener {
                override fun onStart() {
                    channel.invokeMethod("verification_start", "Verification started")
                }

                override fun onFinish(identityAccessKey: String) {
                    channel.invokeMethod("verification_complete", identityAccessKey)
                }

                override fun onError(errorCode: String) {
                    channel.invokeMethod("verification_error", errorCode)
                }

                override fun onSubmitted(identityAccessKey: String) {
                    channel.invokeMethod("verification_submitted", identityAccessKey)
                }
            })
            passbase.startVerification()
        }
    }

    override fun dispose() {
    }

    override fun getView(): View {
        return passbaseButton
    }
}
